/*
  "CRCGenerator"
  Library supporting generation of a CRC8 checksum for a given dataset of 1...255 byte using CRC8-polynomial 0x97
  V1.1 / Copyright (C) 2020, T.Rode (DL1DUZ)

  Permission is granted to use, copy, modify, and distribute this software
  and documentation for non-commercial purposes.

  Required ressources:
   - 256 Byte EEPROM space to store lookup table
*/

//################################################################################################################
//includes
//################################################################################################################

#include <CRCGenerator.h>

//################################################################################################################
//declarations
//################################################################################################################

//################################################################################################################
//private functions
//##############################################################################################################

//################################################################################################################
//public functions
//################################################################################################################

/*
initializes the library / the CRC lookup-table will be stored in EEPROM

"LOOKUP_ADDRESS" -> the EEPROM start address to store the 256 Byte lookup table

returns the next free address in EEPROM
*/
uint16_t CRCGeneratorClass::init(const uint16_t LOOKUP_ADDRESS) {

  // ... CRC8 lookup table ...
  lookup_table_address = LOOKUP_ADDRESS;
  uint8_t  dividend = 0;
  uint16_t remainder;

  // Compute the remainder of each possible dividend.
  do {
    // Start with the dividend.
    remainder = dividend;
    // Perform modulo-2 division, a bit at a time.
    for (uint8_t bit = 8; bit > 0; --bit) {
      // Try to divide the current data bit by the CRC8-polynomial 0x97.
      remainder <<= 1;
      if (remainder & 0x100) {
        remainder ^= 0x97;
      }
    }

    // Store the result into the EEPROM.
    EEPROM.update(lookup_table_address+dividend, remainder);

    ++dividend;
  }
  while(dividend);

  return (LOOKUP_ADDRESS+0x100);
}

//################################################################################################################

/*
calculates the CRC8 value for a given dataset

"ID" -> the ID of the dataset
        this is an arbitrary number from 0...255 used to identify individual datasets
		"ID" is part of the CRC-calculation
"DATA" -> pointer to the dataset
"SIZE" -> size of the dataset (1...255 Byte)
*/
uint8_t CRCGeneratorClass::crcCalculation(const uint8_t ID, const uint8_t* DATA, const uint8_t SIZE) {
  // Set initial remainder to 255-ID
  uint8_t remainder = 0xFF-ID;

  // Byte by Byte divide the message by the polynomial.
  for (uint8_t position = 0; position < SIZE; ++position) {
    remainder = EEPROM[(*(DATA+position) ^ remainder) + lookup_table_address] ^ (remainder << 8);
  }

  return (remainder);
}

CRCGeneratorClass CRC;